#ifndef GRAPH_H
#define GRAPH_H
#include <vector>
#include <string>
#include <map>
#include "GraphLoader.h"

//class Arc;
class Graph;
class Arc		    //߽ṹ
{
public:
	int m_iNodeTo;
	int m_iNodeFrom;
	float m_fWeight;
    float m_fOriginWeight;
	float m_Bet;
	Arc* m_pNextArc;
	Arc* m_pImage;

public:
	Arc(int iNodeFrom, int iNodeTo, float fWeight, Arc* pNextArc)
	{

		m_iNodeFrom = iNodeFrom;
		m_iNodeTo=iNodeTo;
		m_fOriginWeight=fWeight;
        m_fWeight = 0.0f;
		m_pNextArc=pNextArc;
		m_Bet=0.0f;
		Arc* m_pImage=0;

	}
	Arc(int iNodeTo, float fWeight, Arc* pNextArc)
	{
		m_iNodeFrom = -1;
		m_iNodeTo=iNodeTo;
		m_fOriginWeight=fWeight;
        m_fWeight = 0.0f;
		m_pNextArc=pNextArc;
		m_Bet=0.0f;
		Arc* m_pImage=0;
	}
	~Arc()
	{
		if(m_pNextArc)
			delete m_pNextArc;
	}
};


class Node           //ṹ
{
public:
	bool	m_bMarked;
	bool    m_bVisited;

	int		m_iNode;
	char	m_szName[16];
	int		m_iDegree;
	int		m_iInDegree;
	Arc*	m_pFirst;

	int		m_iComplex;				//ǰComplex
	float   m_Bet;
	float   m_iWDegree;
	float   m_iInWDegree;
	float   m_iTotalWDegree;
    float   m_fTotalWeight;

public:
	Node(){m_iNode=0;m_pFirst=0;m_iDegree=0;m_szName[0]='\0';m_Bet=0.0f;}

	Node(int iNode,char szName[16])
	{
		strcpy(m_szName,szName);
		m_iNode=iNode;
		m_pFirst=0;
		m_iDegree=0;
		m_bMarked=false;
		m_bVisited=false;
		m_iComplex=0;
		m_iInDegree=0;
		m_Bet=0.0f;
        m_fTotalWeight = 0.0f;
		m_iWDegree=0.0f;
	    m_iInWDegree=0.0f;
	    m_iTotalWDegree=0.0f;
	}

	~Node(){if(m_pFirst) delete m_pFirst;}
	
	void InsertArc(int iTo,float fWeight)
	{
		Arc* newArc=new Arc(iTo,fWeight,m_pFirst);
		m_pFirst=newArc;
	}
	void InsertArc(int iFrom, int iTo,float fWeight)
	{
		Arc* newArc=new Arc(iFrom,iTo,fWeight,m_pFirst);
		m_pFirst=newArc;
	}

};

class Complex
{
public:

	std::vector<int>	m_ComplexNodes;
	int					m_iNumNodes;
	int                 m_iInDegree;
	float               m_iInWDegree;
	int                 m_iTotalDegree;
	float               m_iTotalWDegree;
	int					m_iComplexNo;
	bool				m_bMergeable;
	bool                m_bModule;

	Graph*				m_pParent;
	static   float      m_sThreshold;

public:
	Complex(int iComplexNo, Graph* parent)
	{		
		m_pParent = parent;
		m_iComplexNo = iComplexNo;
		m_iNumNodes = 0;
		m_iInDegree = 0;
		m_iInWDegree = 0.0f;
		m_iTotalDegree = 0;
		m_iTotalWDegree = 0.0f;
		m_bMergeable = true;
		m_bModule = false;
	}

	void Merge(Complex* pComplex);

};



class Graph          //ڽбʾͼṹ
{

public:
	std::vector<Node*>	m_NodeArray;             //ڵ
	int					m_nNumEdges;
	std::vector<Complex*> m_ComplexArray;

	char				m_szFileName[120];
	std::string			m_strFileName;


public:
	Graph(const char* szFileName);   //ʼͼϢ

	void LoadGraphFromRawGraph(GraphLoader&	rawGraph);

	void FilterGraph(int iDegreeThreshold);

	void ComputeBetweenness();

	void ComputeArcWeight();

	void ShowGraph();
	
	void ClearArcBet()
	{
		for(size_t i=0; i<m_NodeArray.size(); ++i)
		{
			Arc* pArc = m_NodeArray[i]->m_pFirst;
			while(pArc!=NULL)
			{
				pArc->m_Bet = 0.0f;
				pArc = pArc->m_pNextArc;
			}
		}

	}

	void ClearNodeBet()
	{
		for(size_t i=0; i<m_NodeArray.size(); ++i)
		{
			m_NodeArray[i]->m_Bet=0.0f;
		}
	}
	
	void ClearMark()
	{
		for(size_t i=0;i<m_NodeArray.size();++i)
		{
			m_NodeArray[i]->m_bMarked=false;
		}
	}

	void ClearVisit()
	{
		for(size_t i=0;i<m_NodeArray.size();++i)
		{
			m_NodeArray[i]->m_bVisited=false;
		}
	}


	void ClearNodes()
	{
		for(size_t i=0;i<m_NodeArray.size();++i)
		{
			if(m_NodeArray[i]!=0)
				delete m_NodeArray[i];
			m_NodeArray[i]=0;
		}
		m_NodeArray.clear();
	}

	void ClearComplex()
	{
		for(size_t i=0;i<m_ComplexArray.size();++i)
		{
			if(m_ComplexArray[i]!=0)
				delete m_ComplexArray[i];
			m_ComplexArray[i]=0;
		}
		m_ComplexArray.clear();
	}

	Node* GetNode(int iIndex)		//iIndexΪͼнڵı
	{
		return m_NodeArray[iIndex];		//ıǴ0ʼ
	}
	~Graph()
	{
		ClearNodes();
		ClearComplex();
	}


	void DumpComplexInfo(const char* szFileName,int iSize);
	void DumpNodeInfo(const char* szFileName);
	void GenerateComplexBy(float fThreshold); 
};


class Clique
{
public:
	std::vector<int>	m_CliqueNodes;
	int					m_iNumNodes;
	bool				m_bHasMerged;
	int					m_iCliqueID;

	std::map<int, float> m_Dependency;
	int					m_iMaxPiM;

private:
	//std::vecror<int>	m_TempNodes;
public:
	Clique()
	{
		m_iNumNodes=0;
		m_bHasMerged=false;
		m_iCliqueID=-1;
	}

	void sortNodes();
};





#endif
